import { Browser, BrowserContext, Page } from 'puppeteer';
import { CreateOptions } from '../model/interface';
/**
 * Function to create a new tab in the browser instance
 * @param Browser - Browser instance to create a new tab
 * @param options - Options to create a new session in the browser instance
 * @returns - Returns a new tab or false if it fails
 */
export declare function initBrowser(Browser: Browser, options: CreateOptions, ev: any): Promise<Page | boolean>;
/**
 * function to listen for logout events in the browser page
 * @param page - The page instance where the listener will be set
 * @param ev - The event emitter to emit status updates
 * @param options = - Options for creating a new session in the browser instance
 */
export declare const logoutListener: (page: Page, ev: any, options: CreateOptions) => Promise<void>;
/**
 * Function to create a new session in the browser instance
 * @param options - Options to create a new session in the browser instance
 */
export declare function PathSession(options: CreateOptions): void;
/**
 * Function to create a new session in the browser instance
 * @param options - Options to create a new session in the browser instance
 * @param ev - Event to emit status
 * @returns - Returns a new browser instance or false if it fails
 */
export declare function initLaunch(options: CreateOptions, ev: any): Promise<Browser | boolean>;
/**
 * Function to close all extra pages in the browser instance
 * @param browser - Browser instance to close extra pages
 */
export declare function closeExtraPages(browser: Browser | BrowserContext): Promise<void>;
/**
 * Function to create a new tab in the browser instance
 * @param Browser - Browser instance to create a new tab
 * @returns - Returns a new tab or false if it fails
 */
export declare function oneTab(Browser: Browser | BrowserContext): Promise<Page | any>;
/**
 * Function to get the path of the Chrome browser
 * @return - Returns the path of the Chrome browser or undefined if it fails
 * */
export declare function getPathChrome(): string | undefined;
/**
 * ListenerLayer class to handle events in the browser page
 * This class uses an EventEmitter to listen for events and expose functions to the page.
 */
export declare class Listener {
    page: Page;
    constructor(page: Page);
    private listenerEmitter;
    /**
     * @param functionName - The name of the function to listen for events
     * @param fn - Optional callback function to handle the event
     * This method checks if the function is already defined in the page context,
     */
    getListenerEmitter(functionName: string, fn?: (info: any) => void): Promise<{
        dispose: () => void;
    } | undefined>;
}
