"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Listener = exports.getPathChrome = exports.oneTab = exports.closeExtraPages = exports.initLaunch = exports.PathSession = exports.logoutListener = exports.initBrowser = void 0;
const puppeteer_extra_1 = __importDefault(require("puppeteer-extra"));
const ChromeLauncher = __importStar(require("chrome-launcher"));
const path = __importStar(require("path"));
const fs = __importStar(require("fs"));
const enum_1 = require("../model/enum");
const help_1 = require("../help");
const events_1 = require("events");
const StealthPlugin = require('puppeteer-extra-plugin-stealth');
puppeteer_extra_1.default.use(StealthPlugin());
/**
 * Function to create a new tab in the browser instance
 * @param Browser - Browser instance to create a new tab
 * @param options - Options to create a new session in the browser instance
 * @returns - Returns a new tab or false if it fails
 */
async function initBrowser(Browser, options, ev) {
    await closeExtraPages(Browser);
    const wpage = await oneTab(Browser);
    if (wpage) {
        try {
            await wpage.setUserAgent(help_1.puppeteerConfig.useragentOverride);
            await wpage.setCacheEnabled(true);
            const puppeteerOpts = options.puppeteerOptions;
            const proxyUsername = puppeteerOpts === null || puppeteerOpts === void 0 ? void 0 : puppeteerOpts.proxyUsername;
            const proxyPassword = puppeteerOpts === null || puppeteerOpts === void 0 ? void 0 : puppeteerOpts.proxyPassword;
            const listProxy = puppeteerOpts === null || puppeteerOpts === void 0 ? void 0 : puppeteerOpts.listProxy;
            const hasProxyCredentials = typeof proxyUsername === 'string' &&
                proxyUsername.length > 0 &&
                typeof proxyPassword === 'string' &&
                proxyPassword.length > 0;
            const hasProxyList = Array.isArray(listProxy) && listProxy.length > 0;
            // If proxy credentials and list are provided, set the proxy
            if (hasProxyCredentials && hasProxyList) {
                await wpage.authenticate({
                    username: proxyUsername,
                    password: proxyPassword,
                });
            }
            const response = await wpage.goto(help_1.puppeteerConfig.whatsappUrl, {
                waitUntil: 'domcontentloaded',
            });
            if (response && response.status() === 407) {
                return false;
            }
            wpage.on('pageerror', ({ message }) => {
                const erroLogType1 = message.includes('RegisterEffect is not a function');
                const erroLogType2 = message.includes('[Report Only]');
                if (erroLogType1 || erroLogType2) {
                    wpage.evaluate(() => {
                        localStorage.clear();
                        window.location.reload();
                    });
                }
            });
            (0, exports.logoutListener)(wpage, ev, options);
            Browser.userAgent();
            return wpage;
        }
        catch (e) {
            console.error('Error initializing browser:', e);
            return false;
        }
    }
    return false;
}
exports.initBrowser = initBrowser;
/**
 * function to listen for logout events in the browser page
 * @param page - The page instance where the listener will be set
 * @param ev - The event emitter to emit status updates
 * @param options = - Options for creating a new session in the browser instance
 */
const logoutListener = async (page, ev, options) => {
    const event = new Listener(page);
    await event.getListenerEmitter('eventOnLogout', (data) => {
        ev.emitStatusFind({
            error: false,
            text: data.text,
            status: data.type,
            statusFind: 'page',
            onType: enum_1.OnMode.connection,
            session: options.session,
        });
    });
    await page
        .evaluate(async () => {
        let WAWebCmd = undefined;
        if (window.WAWebCmdInitialized)
            return;
        window.WAWebCmdInitialized = true;
        const waitForCmd = async () => {
            await new Promise((resolve) => {
                const check = () => {
                    var _a;
                    WAWebCmd = window.importNamespace
                        ? window.importNamespace('WAWebCmd')
                        : {};
                    if ((_a = WAWebCmd === null || WAWebCmd === void 0 ? void 0 : WAWebCmd.Cmd) === null || _a === void 0 ? void 0 : _a.on) {
                        resolve();
                    }
                    else {
                        setTimeout(check, 200);
                    }
                };
                check();
            });
        };
        await waitForCmd();
        console.log('WAWebCmd is ready');
        WAWebCmd.Cmd.on('starting_logout', () => {
            if (typeof window.eventOnLogout === 'function') {
                window.eventOnLogout({
                    type: 'starting_logout',
                    text: 'Logout is in progress: the user is actively logging out.',
                });
            }
        });
        WAWebCmd.Cmd.on('logout', () => {
            if (typeof window.eventOnLogout === 'function') {
                window.eventOnLogout({
                    type: 'logout',
                    text: 'Logout completed: the user is no longer connected.',
                });
            }
        });
    })
        .catch((e) => {
        console.log('Error in logoutListener:', e);
    });
};
exports.logoutListener = logoutListener;
/**
 * Function to create a new session in the browser instance
 * @param options - Options to create a new session in the browser instance
 */
function PathSession(options) {
    options.mkdirFolderToken = options.mkdirFolderToken
        ? options.mkdirFolderToken
        : '';
    options.pathNameToken = options.pathNameToken ? options.pathNameToken : '';
    options.session = options.session ? options.session : '';
    const folderNameToken = path.join(path.resolve(process.cwd(), options.mkdirFolderToken, options.pathNameToken));
    if (!fs.existsSync(folderNameToken)) {
        fs.mkdirSync(folderNameToken, { recursive: true });
    }
    const pathSession = path.join(path.resolve(process.cwd(), options.mkdirFolderToken, options.pathNameToken, options.session));
    if (!fs.existsSync(pathSession)) {
        fs.mkdirSync(pathSession, { recursive: true });
    }
    fs.chmodSync(folderNameToken, '777');
    fs.chmodSync(pathSession, '777');
    if (options && options.puppeteerOptions) {
        options.puppeteerOptions.userDataDir = pathSession;
    }
    help_1.puppeteerConfig.chromiumArgs.push(`--user-data-dir=${pathSession}`);
}
exports.PathSession = PathSession;
/**
 * Function to create a new session in the browser instance
 * @param options - Options to create a new session in the browser instance
 * @param ev - Event to emit status
 * @returns - Returns a new browser instance or false if it fails
 */
async function initLaunch(options, ev) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l;
    PathSession(options);
    if (((_a = options.puppeteerOptions) === null || _a === void 0 ? void 0 : _a.executablePath) === 'useChrome') {
        const chromePath = getPathChrome();
        options.puppeteerOptions.executablePath = chromePath;
    }
    if (((_b = options.puppeteerOptions) === null || _b === void 0 ? void 0 : _b.downloadChromium) &&
        ((_c = options.puppeteerOptions) === null || _c === void 0 ? void 0 : _c.chromiumVersion) &&
        ((_d = options.puppeteerOptions) === null || _d === void 0 ? void 0 : _d.executablePath) === 'useChromium') {
        const browserFetcher = puppeteer_extra_1.default.createBrowserFetcher({});
        let init = true;
        ev.emitStatusFind({
            error: false,
            text: 'Await download Chromium',
            status: enum_1.TypeStatusFind.chromium,
            statusFind: 'browser',
            onType: enum_1.OnMode.connection,
            session: options.session,
        });
        await browserFetcher
            .download((_e = options.puppeteerOptions) === null || _e === void 0 ? void 0 : _e.chromiumVersion, (downloadedByte, totalBytes) => {
            if (init) {
                ev.emitStatusFind({
                    error: false,
                    text: 'Checking the total bytes to download!',
                    status: enum_1.TypeStatusFind.chromium,
                    statusFind: 'browser',
                    onType: enum_1.OnMode.connection,
                    session: options.session,
                });
                if (totalBytes) {
                    ev.emitStatusFind({
                        error: false,
                        text: `Total Bytes ${totalBytes}`,
                        status: enum_1.TypeStatusFind.chromium,
                        statusFind: 'browser',
                        onType: enum_1.OnMode.connection,
                        session: options.session,
                    });
                }
                init = true;
            }
            if (downloadedByte) {
                ev.emitStatusFind({
                    error: false,
                    text: `Total Bytes: ${totalBytes} download: ${downloadedByte}`,
                    status: enum_1.TypeStatusFind.chromium,
                    statusFind: 'browser',
                    onType: enum_1.OnMode.connection,
                    session: options.session,
                });
            }
            if (downloadedByte === totalBytes) {
                ev.emitStatusFind({
                    error: false,
                    text: `Extract files... await...`,
                    status: enum_1.TypeStatusFind.chromium,
                    statusFind: 'browser',
                    onType: enum_1.OnMode.connection,
                    session: options.session,
                });
            }
        })
            .then((revisionInfo) => {
            var _a, _b;
            if ((_a = options.puppeteerOptions) === null || _a === void 0 ? void 0 : _a.executablePath) {
                options.puppeteerOptions.executablePath =
                    revisionInfo === null || revisionInfo === void 0 ? void 0 : revisionInfo.executablePath;
                ev.emitStatusFind({
                    error: false,
                    text: `download completed, path: ${revisionInfo === null || revisionInfo === void 0 ? void 0 : revisionInfo.executablePath}`,
                    status: enum_1.TypeStatusFind.chromium,
                    statusFind: 'browser',
                    onType: enum_1.OnMode.connection,
                    session: options.session,
                });
            }
            if ((_b = options.puppeteerOptions) === null || _b === void 0 ? void 0 : _b.args) {
                options.puppeteerOptions.args.push(`--single-process`);
            }
        })
            .catch((e) => {
            ev.emitStatusFind({
                error: true,
                text: `Error chromium`,
                status: enum_1.TypeStatusFind.chromium,
                statusFind: 'browser',
                onType: enum_1.OnMode.connection,
                session: options.session,
                result: e,
            });
        });
    }
    if (!((_f = options.puppeteerOptions) === null || _f === void 0 ? void 0 : _f.executablePath)) {
        return false;
    }
    const puppeteerOpts = options.puppeteerOptions;
    const listProxy = puppeteerOpts === null || puppeteerOpts === void 0 ? void 0 : puppeteerOpts.listProxy;
    if (Array.isArray(listProxy) && listProxy.length > 0) {
        const proxy = listProxy[Math.floor(Math.random() * listProxy.length)];
        const proxyArg = `--proxy-server=${proxy}`;
        const passArgs = puppeteerOpts.args;
        if (Array.isArray(passArgs) && passArgs.length > 0) {
            options.puppeteerOptions.args = [...passArgs, proxyArg];
        }
        else {
            help_1.puppeteerConfig.chromiumArgs = [
                ...help_1.puppeteerConfig.chromiumArgs,
                proxyArg,
            ];
        }
    }
    try {
        return await puppeteer_extra_1.default.launch({
            headless: (_g = options.puppeteerOptions) === null || _g === void 0 ? void 0 : _g.headless,
            args: [
                ...help_1.puppeteerConfig.chromiumArgs,
                ...(((_h = options.puppeteerOptions) === null || _h === void 0 ? void 0 : _h.args) || []),
            ],
            executablePath: (_j = options.puppeteerOptions) === null || _j === void 0 ? void 0 : _j.executablePath,
            userDataDir: (_k = options.puppeteerOptions) === null || _k === void 0 ? void 0 : _k.userDataDir,
            devtools: (_l = options.puppeteerOptions) === null || _l === void 0 ? void 0 : _l.devtools,
        });
    }
    catch (e) {
        return false;
    }
}
exports.initLaunch = initLaunch;
/**
 * Function to close all extra pages in the browser instance
 * @param browser - Browser instance to close extra pages
 */
async function closeExtraPages(browser) {
    const pages = await browser.pages();
    for (let i = 1; i < pages.length; i++) {
        const page = pages[i];
        if (page.isClosed())
            continue;
        try {
            await page.close();
        }
        catch (_a) {
            continue;
        }
    }
}
exports.closeExtraPages = closeExtraPages;
/**
 * Function to create a new tab in the browser instance
 * @param Browser - Browser instance to create a new tab
 * @returns - Returns a new tab or false if it fails
 */
async function oneTab(Browser) {
    try {
        const page = await Browser.pages();
        if (page.length) {
            return page[0];
        }
        return await Browser.newPage();
    }
    catch (_a) {
        return false;
    }
}
exports.oneTab = oneTab;
/**
 * Function to get the path of the Chrome browser
 * @return - Returns the path of the Chrome browser or undefined if it fails
 * */
function getPathChrome() {
    try {
        const chromeInstalations = ChromeLauncher.Launcher.getInstallations();
        return chromeInstalations[0];
    }
    catch (_a) {
        return undefined;
    }
}
exports.getPathChrome = getPathChrome;
/**
 * ListenerLayer class to handle events in the browser page
 * This class uses an EventEmitter to listen for events and expose functions to the page.
 */
class Listener {
    constructor(page) {
        this.page = page;
        this.listenerEmitter = new events_1.EventEmitter();
    }
    /**
     * @param functionName - The name of the function to listen for events
     * @param fn - Optional callback function to handle the event
     * This method checks if the function is already defined in the page context,
     */
    async getListenerEmitter(functionName, fn) {
        try {
            const hasFunction = await this.page
                .evaluate((funcName) => typeof window[funcName] === 'function', functionName)
                .catch(() => false);
            if (!hasFunction) {
                // Expose the function to the page
                await this.page
                    .exposeFunction(functionName, (...args) => {
                    this.listenerEmitter.emit(functionName, ...args);
                })
                    .catch(() => undefined);
            }
            this.listenerEmitter.on(functionName, (event) => {
                fn && fn(event);
            });
            return {
                dispose: () => {
                    this.listenerEmitter.off(functionName, (event) => {
                        fn && fn(event);
                    });
                },
            };
        }
        catch (error) {
            console.error('Error in getListenerEmitter:', error);
        }
    }
}
exports.Listener = Listener;
